#!/usr/bin/env ruby

=begin
WPScan - WordPress Security Scanner
Copyright (C) 2011  Ryan Dewhurst AKA ethicalhack3r

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ryandewhurst at gmail
=end

class Validate

  attr_accessor :url, :username, :wordlist

  def initialize(url,username,wordlist)
    @url = url
    @username = username
    @wordlist = wordlist
  end

  # Ensures the url is in the correct format

  def url

    @url = add_slash(@url)
    @url = add_protocol(@url)
    
    begin
      @url = URI.parse(@url)
    rescue => e
      puts
      puts 'Validation ERROR: URL not in correct format... ' + e.inspect
      puts @url.inspect
      exit(1)      
    end

  end

  # Add a leading slash to a url
  
  def add_slash(url)
    if url !~ /\/$/
      url = url + '/'
    else
      url = url
    end 
  end

  # Remove a leading slash from a url 

  def remove_slash(url)
    url = url.to_s
    if url =~ /\/$/
      url = url.gsub!(/\/$/,'')
    else
      url = url
    end
  end

  # Add protocol

  def add_protocol(url)
    if url !~ /^https?:/
      url = 'http://' + url
    else
      url = url
    end
  end

  # does the wordlist file exist?
  
  def wordlist
    if @wordlist != ''
      if !File.exists? @wordlist
        return false
      end
    end
  end

  # Checks if the remote website is up.
  #
  # checks to see if the remote website,
  # is responding to our requests by,
  # first trying a DNS request, if that fails,
  # then try a reverse DNS lookup, if that,
  # failes, we assume the remote host is down.

  def website_up?(url)
    up = true
    url = URI.parse(remove_slash(url))

    begin
      # send a dns request 
      Resolv.getaddress(url.host)
    rescue
      begin
        # send a reverse dns request
        Resolv.getname(url.host) 
      rescue
        up = false
      end 
    end

    return up
  end

end
