#!/usr/bin/env ruby

=begin
WPScan - WordPress Security Scanner
Copyright (C) 2011  Ryan Dewhurst AKA ethicalhack3r

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ryandewhurst at gmail
=end

# This tool generates a plugin list to use for plugin enumeration

class Generate_Plugin_List

  attr_accessor :pages, :threads, :verbose

  def initialize(pages, threads, verbose)
    @pages = pages.to_i
    @threads = threads
    @verbose = verbose
  end

  # Send a HTTP request to the WordPress most popular plugins webpage
  # parse the response for the plugin names. 

  def parse_plugins

    found_plugins = ""
    page_count = 1
    queue_count = 0

    hydra = Typhoeus::Hydra.new(:max_concurrency => @threads, :timeout => 10000)
    
    (1...@pages).each do |page|

      request = Typhoeus::Request.new('http://wordpress.org/extend/plugins/browse/popular/page/'+page.to_s+'/')
      queue_count += 1
    
      request.on_complete do |response|
        puts "[+] Parsing page " + page_count.to_s if @verbose
        page_count += 1
        response.body.scan(%r{<h3><a href="http://wordpress.org/extend/plugins/(.*)/">.+</a></h3>}i).each do |plugin|
          found_plugins << plugin[0] + "\n" 
        end
      end

      hydra.queue(request)

      if queue_count == @threads
         hydra.run
         queue_count = 0
       end

    end

    hydra.run

    found_plugins
  end

  # Save the file

  def save_file
    begin
      plugins = parse_plugins
      File.open('data/plugins.txt', 'w') { |f| f.write(plugins) }
      puts "New data/plugin.txt file created with " + plugins.scan(/\n/).size.to_s + " entries." 
    rescue => e
      puts "ERROR: Something went wrong :( " + e.inspect
    end
  end
 
end
