#!/usr/bin/env ruby

=begin
WPScan - WordPress Security Scanner
Copyright (C) 2011  Ryan Dewhurst AKA ethicalhack3r

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ryandewhurst at gmail
=end

$: << '.'

begin
  require 'rubygems'
  require 'getoptlong'
  require 'uri'
  require 'time'
  require 'resolv'
  require 'lib/discover'
  require 'lib/bruter'
  require 'lib/validate'
  require 'tools/generate_plugin_list'
  require 'typhoeus'
  require 'xmlsimple'
rescue LoadError => e
  puts "ERROR: Missing ruby gem. Please see README file."
  puts e.inspect
end

# Declare needed variables

url = ''
enum_version = false
enum_plugins = false
enum_users = false
generate_plugin_list = 0
username = ''
wordlist = ''
password = ''
threads = 20
verbose = false
@version = "1.0"

# our 1337 banner

def banner()
  puts '____________________________________________________'
  puts " __          _______   _____                  "
  puts " \\ \\        / /  __ \\ / ____|                 "
  puts "  \\ \\  /\\  / /| |__) | (___   ___  __ _ _ __  "
  puts "   \\ \\/  \\/ / |  ___/ \\___ \\ / __|/ _` | '_ \\ "
  puts "    \\  /\\  /  | |     ____) | (__| (_| | | | |"
  puts "     \\/  \\/   |_|    |_____/ \\___|\\__,_|_| |_|" + " v" + @version
  puts
  puts "  WordPress Security Scanner by ethicalhack3r.co.uk"
  puts " Sponsored by the RandomStorm Open Source Initiative"
  puts '_____________________________________________________'
  puts
end

# GNU license

def license()
  puts "# Copyright (C) 2011 Ryan Dewhurst"
  puts "# This program comes with ABSOLUTELY NO WARRANTY."
  puts "# This is free software, and you are welcome to redistribute it"
  puts "# under certain conditions. See GNU GPLv3."
end

# for the noobs

def usage()
  script_name = $0 
  puts
  puts "-h for further help."
  puts 
  puts "Examples:"
  puts
  puts "Do 'non-intrusive' checks..."
  puts "ruby " + script_name + " --url www.example.com"
  puts
  puts "Do wordlist password brute force on enumerated users using 50 threads..."
  puts "ruby " + script_name + " --url www.example.com --wordlist darkc0de.lst --threads 50"
  puts
  puts "Do wordlist password brute force on the 'admin' username only..."
  puts "ruby " + script_name + " --url www.example.com --wordlist darkc0de.lst --username admin"
  puts
  puts "Generate a new 'most popular' plugin list, up to 150 pages..."
  puts "ruby " + script_name + " --generate_plugin_list 150"
  puts
  puts "Enumerate instaled plugins..."
  puts "ruby " + script_name + " --enumerate p"
  puts 
  puts 'See README for further information.'
end

# command help

def help()
  puts "Help:"
  puts
  puts "--url				The WordPress URL/domain to scan."
  puts "--enumerate			Enumeration."
  puts "u				users"
  puts "v				version"
  puts "p				plugins"
  puts "--wordlist			Supply a wordlist for the password bruter and do the brute."
  puts "--threads			The number of threads to use when multi-threading requests."
  puts "--username			Only brute force the supplied username."
  puts "--generate_plugin_list		Generate a new data/plugins.txt file. (supply number of *pages* to parse)"
  puts "-h				This help screen."
  puts "-v				Verbose output."
  puts
end

# Output banner

banner()
license()
puts

# do we have any input?!

if ARGV.length == 0
  usage()
  exit(1)
end

# Deal with user supplied input

opts = GetoptLong.new(
  ["--url", GetoptLong::OPTIONAL_ARGUMENT],
  ["--enumerate", GetoptLong::OPTIONAL_ARGUMENT],
  ["--generate_plugin_list", GetoptLong::OPTIONAL_ARGUMENT],
  ["--username", GetoptLong::OPTIONAL_ARGUMENT],
  ["--wordlist", GetoptLong::OPTIONAL_ARGUMENT],
  ["--threads", GetoptLong::OPTIONAL_ARGUMENT],
  ["-h", GetoptLong::OPTIONAL_ARGUMENT],
  ["-v", GetoptLong::OPTIONAL_ARGUMENT] 
) 

begin

  opts.each do |opt, arg|

    case opt

      when '--url'
        url = arg
      when '--enumerate'
	if arg == "v"
          enum_version = true
        elsif arg == "p"
          enum_plugins = true
        elsif arg == "u"
          enum_users = true
        else
          puts "ERROR: Unknown enumerate option. v = version, p = plugins"
          exit(1)
        end
      when '--generate_plugin_list'
        if arg == ""
          puts "Number of pages not supplied, defaulting to 150 pages..."
          puts
          generate_plugin_list = 150
        else
          generate_plugin_list = arg
        end
      when '--username'
        username = arg
      when '--wordlist'
        wordlist = arg
      when '--threads'
          threads = arg.to_i
      when '-h'
          help()
          exit(1)
      when 
        verbose = true 
      end

   end

rescue

  banner()
  usage()
  help()
  exit(1)

end

# Generates a new plugin list for plugin enumeration

unless generate_plugin_list == 0
  puts "[+] Generating new most popular plugin list"
  puts
  Generate_Plugin_List.new(generate_plugin_list,threads,verbose).save_file
  exit(1) if url == ""
end

# Validate user input...

validate = Validate.new(url,username,wordlist)
url = validate.url

# Wordlist exists?

if validate.wordlist == false
  puts "ERROR: File " + wordlist.to_s + " does not exist."
  exit(1)
end

# Remote website up?

if validate.website_up?(url) == false
  puts 'ERROR: The remote website ' + url.to_s  + ' seems to be down.' 
  exit(1)
end

# Output runtime data

puts '| URL: ' + url.to_s
puts '| Started on ' + Time.now.asctime
puts 

# Create the discover object

discover = Discover.new(url, threads, verbose)

# Can we identify the theme name?

theme_name = discover.theme_name
unless theme_name == false or theme_name == '' or theme_name.nil?
  puts "[+] The WordPress theme in use is called " + theme_name.to_s
end

# Is the readme.html file there?

if discover.readme_exists?
  puts "[+] The WordPress " + url.to_s  + "readme.html file exists."
end

# Is there a error_log file in te plugins dir?

if discover.error_log
  puts "[+] A WordPress error_log file has been found: " + url.to_s + "wp-content/plugins/error_log"
end

# If the html generator meta tag is in the source, extract the version...
gen_version = discover.generator_version
unless gen_version == false or gen_version == '' or gen_version.nil?
  puts '[+] WordPress version ' + gen_version.to_s  + ' identified from meta generator.'
end

# Are there any vulnerabilities associated with this version?

unless gen_version == false or gen_version == '' or gen_version.nil?

  wp_vulnerabilities = discover.wp_vulnerabilities(gen_version)

  unless wp_vulnerabilities.nil? or wp_vulnerabilities == ''
    count = 0
    puts '[+] We have identified ' + wp_vulnerabilities.size.to_s  + ' vulnerabilities from the version number: '
    wp_vulnerabilities.each do |vulnerability|
      puts
      puts '* Title: ' + vulnerability[1][count]['title'][0]
      puts '* Reference: ' + vulnerability[1][count]['reference'][0]
      count += 1
    end
  end

end

# If enum_plugins = true, enumerate the installed plugins

if enum_plugins

  puts
  puts "[+] Enumerating installed plugins..."

  plugins = discover.plugins
  unless plugins.empty?

    puts
    puts "We found " + plugins.size.to_s  + " plugins:"
    puts  
    puts plugins.flatten

    # are there any vulnerabilities associated with the found plugins?
    plugin_vulns = discover.plugin_vulnerabilities(plugins)
    unless plugin_vulns.nil? or plugin_vulns == ''
      puts
      puts '[+] There were ' + plugin_vulns.size.to_s  + ' vulnerabilities identified from the plugin names:'
      plugin_vulns.each do |vulnerability|
        puts
        puts '* Title: ' + vulnerability['vulnerability'][0]['title'][0]
        puts '* Reference: ' + vulnerability['vulnerability'][0]['reference'][0]
      end
    end

    if discover.directory_listing?(plugins[0].to_s)
      puts
      puts "Directory listing enabled: " + url.to_s + "wp-content/plugins/" + plugins[0].to_s + "/"
    end

  else
    puts
    puts "No plugins found :("
    puts
  end

end

# If we haven't been supplied a username, enumerate them...

if username == '' and wordlist != '' or enum_users == true

  puts
  puts '[+] Enumerating usernames...'

  usernames = discover.usernames

  if usernames.empty?
    puts 
    puts "We did not enumerate any usernames :("
    puts "Try supplying your own username with the --username option"
    puts
    exit(1) 
  else 
    puts 
    puts 'We found the following ' + usernames.length.to_s + ' username/s:'
    puts
    puts usernames
  end

else 

  usernames = [username]

end

# Start the brute forcer

if wordlist != ''

  # Is there is a login-lockdown plugin enabled?
  if discover.login_lockdown
    puts
    puts "[+] ERROR:: login-lockdown plugin enabled. It will records the IP and timestamp of every failed login. Not a good idea for brute forcing!"
    exit(1)
  end

  puts
  puts '[+] Starting the password brute forcer'
  puts
  Bruter.new(url,usernames,wordlist,threads,verbose).brute
end

puts
puts '[+] Finished at ' + Time.now.asctime
